#pragma once

#include <algorithm>
#include <vector>
#include <iostream>
#include <math.h>
#include <ros/node_handle.h>
#include <tf/LinearMath/Quaternion.h>
#include <tf/transform_datatypes.h>
#include <tf/LinearMath/Transform.h>
#include <sensor_msgs/Image.h>
#include "node_aruco/tools/ImageLoader.h"
#include <opencv2/aruco.hpp>
#include <opencv2/opencv.hpp>
#include <eigen3/Eigen/Dense>
#include <eigen3/Eigen/Geometry>
#include <ros/time.h>

using namespace std;

//#include <ecl/linear_algebra/eigenvalues.hpp>

// todo: Use same format as arucoPose.msg
struct avTag{
	int VehId;
	Eigen::Quaternion<double,Eigen::AutoAlign> rVector;
	cv::Vec3d tVector;
};

class ArucoPos
{

public:
	ArucoPos(bool debugging,std_msgs::Header imageHeader,std::string imageEncoding);
	void setParameters(cv::Ptr<cv::aruco::DetectorParameters> param);
	void setDictionary(cv::Ptr<cv::aruco::Dictionary> dict);
	cv_bridge::CvImage getOutputImage();
	int readParametersFromFile();
	vector<avTag> searchForTag(cv::Mat& currentImage);
	void printImage(ros::Publisher publisher);
	
private:
	cv::Ptr<cv::aruco::DetectorParameters> parameters;
	cv::Ptr<cv::aruco::Dictionary> dictionary;

	cv::Mat cameraMatrix;
	cv::Mat distCoeffs;
	
	int tagsOnVehicle;
	int nrOfVehicles;
	
	bool debug;
	
	cv_bridge::CvImage outputImage;
	
	struct navTag {
		int id;
		float size;
		cv::Vec3d position;
	};
	
	struct vehTag{
		int id;
		vector<navTag> tag;
	};

	struct detTags{
		navTag tag;
		int vehicleId;
		vector<cv::Point2f> corners;
		cv::Vec3d rVector;
		cv::Vec3d tVector;
	};
	
	vector<vehTag> vehicles;
	
	vector<int> markerIds;
	vector< vector<cv::Point2f> > markerCorners;
	vector< vector<cv::Point2f> > rejectedCands;
	
	vector< cv::Vec3d> rVecs;
 	vector< cv::Vec3d> tVecs;
};