#include <node_aruco/tools/ArucoPos.h>
//#include <node_msgs/arucoPose.h>
#include "../../../devel/include/node_msgs/arucoPose.h"


ArucoPos::ArucoPos(bool debugging,std_msgs::Header imageHeader,std::string imageEncoding) : parameters(cv::aruco::DetectorParameters::create()), dictionary(cv::aruco::getPredefinedDictionary(cv::aruco::DICT_5X5_50))
{
	debug=debugging;
	
	outputImage.header   = imageHeader;
	outputImage.encoding = imageEncoding;

	tagsOnVehicle = 0;
	nrOfVehicles = 0;
}

void ArucoPos::setParameters(cv::Ptr<cv::aruco::DetectorParameters> param)
{
	parameters = param;
}

void ArucoPos::setDictionary(cv::Ptr<cv::aruco::Dictionary> dict)
{
	dictionary=dict;
}

cv_bridge::CvImage ArucoPos::getOutputImage()
{
	return outputImage;
}

int ArucoPos::readParametersFromFile()
{
	/*
	* 	Read parameters from .yaml files
	*/
	vector<float> tempVector;
	distCoeffs.create(4, 1, CV_32F);
	ros::NodeHandle parameterNode;
	if(parameterNode.getParam("/distortion_coefficients/data",tempVector))
	{
		for(int i=0;i<4;i++){
			distCoeffs.at<float>(i,0) = tempVector[i];
		}
		if(debug) cout << "distortion_coefficients = "<< endl << " "  << distCoeffs << endl << endl;
	}
	else
	{
		ROS_WARN("Erorr loading camera distortion coefficients matrix");
	}

	cameraMatrix.create(3,3, CV_32F);
	if(parameterNode.getParam("/camera_matrix/data",tempVector))
	{
		for(int i=0;i<9;i++){
			//cout << tempVector[i] << endl;
			cameraMatrix.at<float>(int(floor(i/3)),i%3) = tempVector[i];
			//cout << i << ":" << int(floor(i/3)) << "," << i%3 << endl;
		}
		if(debug) cout << "camera_matrix = "<< endl << " "  << cameraMatrix << endl << endl;
	}
	else
	{
		ROS_WARN("Error loading camera matrix!");
	}
	
	
	// Read out .yaml parameters for the navigation tags
	
	if(parameterNode.getParam("/arucoParameters/nrOfVehicles",nrOfVehicles))
	{
		if(debug) cout << "Nr of vehicles: " << nrOfVehicles << endl;
	}
	else
	{
		nrOfVehicles=0;
		ROS_WARN("Error loading nr of vehicles from parameter file");
	}
	
	stringstream navstring;
	int tempNrOfTags;
	vehTag tempVeh;
	navTag tempNav;

	for(int i=0;i<nrOfVehicles;i++)
	{
		tempVeh.id=i;
		vehicles.push_back(tempVeh);
		navstring << "/arucoParameters/vehicle" << i << "/nrOfTags";
		if(parameterNode.getParam(navstring.str(),tempNrOfTags))
		{
			if(debug) cout << "Nr of tags on vehicle #" << i << ": " << tempNrOfTags << endl;
		}
		else
		{
			tempNrOfTags =0;
			ROS_ERROR("ARUCO: Error loading nr of tags from :%s",navstring.str().c_str());
		}
		navstring.str("");
		
		for(int j=0;j<tempNrOfTags;j++)
		{
            vector<double> tempPos;
			navstring << "/arucoParameters/vehicle" << i << "/tag" << j;
			if(parameterNode.getParam(navstring.str()+"/id",tempNav.id))
			{
				if(debug) cout << "ID:" << tempNav.id << endl;
			}
			else
			{
				ROS_ERROR("ARUCO: Error loading tag id from: %s",(navstring.str()+"/id").c_str());
			}

			if(parameterNode.getParam(navstring.str()+"/size",tempNav.size))
			{
				if(debug) cout << "size:" << tempNav.size << endl;
			}
			else
			{
				ROS_ERROR("ARUCO: Error loading tag size from: %s",navstring.str().c_str());
			}

			if(parameterNode.getParam(navstring.str()+"/position",tempPos))
			{
                tempNav.position.zeros();
                for(size_t k=0;k<tempPos.size();k++)
                {
                    tempNav.position[k]=tempPos.at(k);
                }
				if(debug) cout << "position:" << tempNav.position << endl << endl;
			}
			else
			{
                tempNav.position.zeros();
				ROS_ERROR("ARUCO: Error loading tag position from: %s",navstring.str().c_str());
			}
			vehicles[i].tag.push_back(tempNav);
			navstring.str("");
		}
	}
	if(vehicles.size()<1)
	{
		ROS_WARN("No navigation tag parameter file found");
	}

	return 0;
	//Todo:
}


vector<avTag> ArucoPos::searchForTag(cv::Mat& currentImage)
 {
	/*
	* 	Detection
	*/

	cv::aruco::detectMarkers(currentImage, dictionary, markerCorners, markerIds, parameters, rejectedCands );
	
	if(debug) ROS_INFO("ARUCO: Detected markers! Number of valid %i number of rejected: %i", (int)markerIds.size(), (int)rejectedCands.size());
	
	outputImage.image = currentImage.clone();
	cv::aruco::drawDetectedMarkers(outputImage.image, markerCorners, markerIds);
	
	
	/*
	* 	Pose estimation
	*/
	vector< vector<cv::Point2f> > cornerOfInterest;
	vector<detTags> detectedTags;
	detTags tempTag;
    vector<avTag> averageTag;
    cv::Vec3d tempVectSum;
	vector<node_msgs::arucoPose> tempMsg;
	// Check if the tag is listed as marker of interest
	for(int i=0; i<(int)vehicles.size();i++)				//For all vehicles
	{
		for(int j=0; j<(int)vehicles[i].tag.size();j++)		//For all tags on vehicle
		{
			for(int k=0; k<(int)markerIds.size(); k++)		//For all detected tags
			{
				if(markerIds[k]==vehicles[i].tag[j].id)		
				{
                    tempTag.tag=vehicles.at(i).tag.at(j);
					tempTag.corners=markerCorners.at(k);
					tempTag.vehicleId=vehicles.at(i).id;

					cornerOfInterest.push_back(markerCorners.at(k));
					cv::aruco::estimatePoseSingleMarkers(cornerOfInterest ,vehicles[i].tag[j].size,cameraMatrix,distCoeffs,rVecs,tVecs);

					tempTag.rVector=rVecs.back();
					tempTag.tVector=tVecs.back();

					detectedTags.push_back(tempTag);

					if(debug)
					{
						cout << "ID: " << detectedTags.back().tag.id << endl;
						cout << "navigationTags.size: " << detectedTags.back().tag.size<< endl;
						cout << "Rotation:" << detectedTags.back().rVector << endl;
						cout << "Translation" << detectedTags.back().tVector << endl << endl;
					}
					cornerOfInterest.clear();
				}
			}
		}

        double area=0;
        double areai=0;

        Eigen::Vector4d q;
        Eigen::Matrix4d Q;

        // Set to zero
        Q.setZero();

        for(size_t l=0; l< detectedTags.size();l++)    //All detected tags
        {
            //Transform to Tag coordinate system
            cv::Mat rotMat;
            cv::Rodrigues(detectedTags.at(l).rVector, rotMat);

            cv::Mat A = rotMat*cv::Mat(detectedTags.at(l).tag.position);
            cv::Vec3d rtm = A.col(0);

            detectedTags.at(l).tVector=detectedTags.at(l).tVector+rtm;

            // Position Averaging, find sum of area
            areai= cv::contourArea(detectedTags.at(l).corners);
            area = area +areai;                                                 //Sum of area
            tempVectSum = tempVectSum+(areai*detectedTags.at(l).tVector);        //Sum of area times translation vector

            // Including averaging of quaternions!
            // Using the Eigen package
            // Converting from euler to quaternions
            Eigen::Quaternion<double, Eigen::AutoAlign> qua;
            qua=Eigen::AngleAxisd(detectedTags.at(l).rVector[0],Eigen::Vector3d::UnitX())
                *Eigen::AngleAxisd(detectedTags.at(l).rVector[1],Eigen::Vector3d::UnitY())
                *Eigen::AngleAxisd(detectedTags.at(l).rVector[2],Eigen::Vector3d::UnitZ());

            q << qua.w(), qua.x(), qua.y(), qua.z();

            Q=Q+areai*q*q.transpose();
        }

        // Position Averaging
        avTag tempAvTag;
        if(detectedTags.size()>0)
        {
            //Averaging Quaternions
            Q=1/area*Q;

			//tempAvTag.rVector = tf::createQuaternionFromRPY(detectedTags.at(0).rVector[0],detectedTags.at(0).rVector[1],detectedTags.at(0).rVector[2]);

			// Todo: Fix bug in quaternion averaging
            //Using the eigensolver
            Eigen::SelfAdjointEigenSolver<Eigen::Matrix4d> eigenSolver(Q);
            if(eigenSolver.info() == Eigen::Success && 1==0)
            {
                //Debug
                //std::cout << "Eigenvalues:" << std::endl << eigenSolver.eigenvalues() << std::endl;
                //std::cout << "Eigenvectors:" << std::endl << eigenSolver.eigenvectors() << std::endl;

				//Find the average quaternion by solving the eigenvalue problem
				Eigen::Quaternion<double , Eigen::AutoAlign> tempQua(eigenSolver.eigenvectors().col(0));
				tempAvTag.rVector = tempQua;
            }
            else
            {
                //ROS_WARN("NAVIGATION: Cannot solve the eigenvalue problem while averaging quaternions!");
				//Get the rotation from the first tag
				tempAvTag.rVector=Eigen::AngleAxisd(detectedTags.at(0).rVector[0],Eigen::Vector3d::UnitX())
						*Eigen::AngleAxisd(detectedTags.at(0).rVector[1],Eigen::Vector3d::UnitY())
						*Eigen::AngleAxisd(detectedTags.at(0).rVector[2],Eigen::Vector3d::UnitZ());
            }

            tempAvTag.VehId=detectedTags.front().vehicleId;
            tempAvTag.tVector=1/area*tempVectSum;           //Average position

            averageTag.push_back(tempAvTag);
            //cout << "Angle: " << averageTag.back().rVector << endl;
            //cv::aruco::drawAxis(outputImage.image, cameraMatrix, distCoeffs, detectedTags.at(0).rVector, tempAvTag.tVector, 0.09);
			Eigen::Vector3d tempVectForDisp = tempAvTag.rVector.normalized().toRotationMatrix().eulerAngles(0,1,2); //XYZ
			cv::Vec3d a(tempVectForDisp.data());
			cv::aruco::drawAxis(outputImage.image, cameraMatrix, distCoeffs, a, tempAvTag.tVector, 0.09);
            //cout << "Tag iD: " << detectedTags.at(0).tag.id << endl;
        }
        detectedTags.clear();
	}
	return averageTag;
}


void ArucoPos::printImage(ros::Publisher publisher)
{
	publisher.publish(outputImage.toImageMsg());
}