//
// Created by line on 28.02.18.
//

#ifndef PROJECT_AUTOPILOT_H
#define PROJECT_AUTOPILOT_H

#include <math.h>
#include <eigen3/Eigen/Dense>
#include <utils/readRosParam.h>

class Autopilot
{
public:
    struct mode{
        static const int stop=0;
        static const int intercept=1;
        static const int hover=2;
        static const int lower=3;
        static const int gainAdjust=4;
        static const int finalStage=5;
        static const int land=6;
        static const int landed=7;
    };

    Autopilot();
    Eigen::Vector3d controller(int md,Eigen::Vector3d p_u_t, Eigen::Vector3d v_n_t, double dt);
    int modeSelector(int md, bool control_given, Eigen::Vector3d p_u_t, Eigen::Matrix3d p_u_t_covar);
    void loadParameterFile();

    bool getEnambleLand();

    double getHoverHeight();

private:
    static Eigen::Vector3d constantBearingController(Eigen::Vector3d p_u_t, Eigen::Vector3d v_n_t,
                                                     double v_u_max, double dp);

    //Parameters
    double v_u_max_;
    double dp_;
    double dp_gainAdjust_;
    double hoverHeight_;
    double decreaseRate_;
    double posTolIntercept_;
    double covarTolhover_;
    double posTolLowerRadius_;
    double posTolLowerHeight_;
    double gainAdjustAlt_;
    double finalStageAlt_;
    double finalStageDecentVel_;

    bool enableLand_;
    double dynhoverHeight_;
    Eigen::Vector3d hoverVect_;
    bool uavLanded;
};


#endif //PROJECT_AUTOPILOT_H