//
// Created by Line on 21.03.18.
//

#ifndef PROJECT_NAVIGATION_ARUCOCALLBACK_H
#define PROJECT_NAVIGATION_ARUCOCALLBACK_H

#include "node_msgs/arucoPose.h"
#include <ros/time.h>
#include <eigen3/Eigen/Dense>
#include "coordinateFrames.h"
#include "kalmanFilter.h"
#include "writeToFile.h"
#include <list>

class ArucoCallback{
public:
    struct pose{
        ros::Time timestamp;            //GNSS time
        Eigen::Vector3d position;       //Relative to NED
        Eigen::Vector3d orientation;
        Eigen::Matrix<double,3,3> covariance_pos;
        Eigen::Matrix<double,3,3> covariance_ori;
    };

    ArucoCallback(kalmanFilter& kf_);
    void poseArucoCallback(const node_msgs::arucoPose::ConstPtr& arucoData);
    pose get_aruco_pose();
    Eigen::Matrix3d calculateCovariance(double height);

    void setDtGnssRos(ros::Duration dtGnssRos_);

private:
    WriteToFile wf;
    bool enableWriteToFile;
    std::ofstream myfile;

    pose aruco_pose;

    double varianceUAVorient;
    double varianceArUcoPos_xy;
    double varianceArUcoPos_height;

    kalmanFilter& kf;

    ros::Duration dtGnssRos;

    //Parameters from parameter file
    Eigen::Matrix3d R_cu;
};

#endif //PROJECT_NAVIGATION_ARUCOCALLBACK_H
