//
// Created by line on 25.01.18.
//


#ifndef NAVIGATION_NODE_KALMANFILTER_H
#define NAVIGATION_NODE_KALMANFILTER_H

#include <eigen3/Eigen/Dense>
#include <eigen3/unsupported/Eigen/EulerAngles>
#include <iostream>
#include <ros/time.h>
#include <ros/init.h>
#include <ros/node_handle.h>
#include <list>

class kalmanFilter {
public:
    struct measurementType{
        static const int arUco_pos=1;
        static const int lP_pos=2;
        static const int lP_vel=3;
    };

    struct bufferElement{
        ros::Time timestamp;
        Eigen::VectorXd element;
        Eigen::VectorXd covariance;
    };

    kalmanFilter();
    kalmanFilter(int m, int n, int o, bool autoInitKf_);

    void setInitStates(const Eigen::MatrixXd& x0_);

    void setInitTime(const ros::Time& timeNow_);

    void projectAhead();

    void updateMeasurement(int type, Eigen::Vector3d& measurement, ros::Time& now);
    void updateMeasurement(int type, Eigen::Vector3d& measurement, ros::Time& now, Eigen::Matrix3d& covariance);

    bool readParametersFromFile();

    void set_u(const Eigen::Vector3d& u);

    void set_UAV_orient(const Eigen::Quaterniond uav_orient,const ros::Time& timeStamp, const Eigen::Vector3d& covariance);
    bufferElement get_UAV_orient(const ros::Time& timestamp);
    bool uav_orient_available();

    void set_UAV_position(const Eigen::Vector3d uav_position,const ros::Time& timeStamp, const Eigen::Vector3d& covariance);
    bufferElement get_UAV_position(const ros::Time& timestamp);
    bool uav_position_available();

    Eigen::MatrixXd getEstStates(ros::Time tNow);

    Eigen::MatrixXd getEstCov();

    void addElementToBuffer(std::list<bufferElement>& buffer, unsigned long bufferSize, const bufferElement& element);

    void testFunctions();

    void setTargetDynamics(bool targetStatic);

    void setStatesInitialized();
    bool getStatesInitialized();

    bool getAutoInit();


private:
    void update_z(int type, Eigen::Vector3d measurement);

    void update_H(int type);

    void update_R(int type, Eigen::Matrix3d covarianse);

    bool check_dt(ros::Time timeNow);

    void deadReckoning(ros::Time& timeNow);

    bool staticTarget_;

    bool autoInitKf_;

    bufferElement findCorrespondingElement(const ros::Time& timestamp, std::list<bufferElement>& buffer);

    //Store last 20 measurements
    std::list<bufferElement> uav_orient_;
    std::list<bufferElement> uav_position_;

    bool debug = false;
    bool timeInitialized =false;
    bool statesInitialized = false;
    //n_->Number of states
    //m_->Number of measurements
    //o_->Number of inputs
    int m_,n_,o_;

    // Identity matrix
    Eigen::MatrixXd I;

    // Measurements, measurements matrix and measurements covariance matrix
    Eigen::VectorXd z;
    Eigen::MatrixXd H;
    Eigen::MatrixXd R;

    // Time variables
    double dt;
    ros::Time prevTime;

    Eigen::MatrixXd Phi, Delta, Q;

    //Kalman gain
    Eigen::MatrixXd K;

    // Estimate error covariance
    Eigen::MatrixXd P, P_pri;

    //Init values from parameter file
    Eigen::MatrixXd P0, x0;

    // States
    Eigen::MatrixXd x_hat, x_hat_pri;

    // Input
    Eigen::VectorXd u_;


    void updatePhiDelta(double dt);

    void updateFilter();
};


#endif //NAVIGATION_NODE_KALMANFILTER_H
