//
// Created by Line on 20.03.18.
//

#ifndef PROJECT_LANDINGPADCALLBACK_H
#define PROJECT_LANDINGPADCALLBACK_H

#include <ros/init.h>
#include <eigen3/Eigen/Dense>
#include <geometry_msgs/PoseWithCovarianceStamped.h>
#include "node_navigation/tools/coordinateFrames.h"
#include <nav_msgs/Odometry.h>
#include <cmath>
#include "kalmanFilter.h"
#include "writeToFile.h"
#include <list>
#include <utils/readRosParam.h>


class LandingPadCallback{
public:
    LandingPadCallback(CoordinateFrames& NED, kalmanFilter& kf_);

    struct pose{
        ros::Time timestamp;            //GNSS time
        Eigen::Vector3d position;       //Relative to NED
        //Eigen::Vector3d orientation;
        Eigen::Quaterniond orientation;
        Eigen::Matrix<double,6,6> covariance;
    };

    struct velocity{
        ros::Time timestamp;            //GNSS time
        Eigen::Vector3d linear;         //Relative to NED
        Eigen::Vector3d angular;
        Eigen::Matrix<double,6,6> covariance;
    };

    void lpPoseVelCallbackOlav(const geometry_msgs::PoseWithCovarianceStamped::Ptr& lpPoseVel);
    void lpPoseVelCallbackControlTower(const nav_msgs::Odometry::Ptr& lpPoseVel);


    pose get_pose();
    velocity get_velocity();

    void setDtGnssRos(ros::Duration dtGnssRos_);
private:
    void initWriter();

    CoordinateFrames& globalNED;

    //Landing pad data
    pose lp_pose;
    velocity lp_vel;

    bool debug = false;

    kalmanFilter& kf;

    WriteToFile wf;
    bool enableWriteToFile;
    std::ofstream myfile;

    //Parameters from parameter file
    Eigen::Vector3d p_lb_l;
    Eigen::Matrix3d R_lb;

    ros::Duration dtGnssRos;
};



#endif //PROJECT_LANDINGPADCALLBACK_H
