//
// Created by Line on 20.03.18.
//

#ifndef PROJECT_UAVCALLBACK_H
#define PROJECT_UAVCALLBACK_H

#include <ros/time.h>
#include <ros/init.h>
#include <sensor_msgs/NavSatFix.h>
#include <sensor_msgs/Imu.h>
#include "node_navigation/tools/coordinateFrames.h"
#include <sensor_msgs/TimeReference.h>
#include <geometry_msgs/Twist.h>
#include <geometry_msgs/TwistStamped.h>
#include "kalmanFilter.h"
#include <mavros_msgs/State.h>

class UavCallback{
public:
    UavCallback(CoordinateFrames& NED,kalmanFilter& kf_);

    struct position{
        ros::Time timestamp;            //GNSS time
        Eigen::Vector3d position;       //Relative to NED
        Eigen::Matrix<double,3,3> covariance;
    };
    struct orientation{
        ros::Time timestamp;            //GNSS time
        //Eigen::Vector3d orientation_euler;    //Relative to NED
        Eigen::Quaterniond orientation;
        Eigen::Matrix<double,3,3> covariance;
    };
    struct velocity{
        ros::Time timestamp;            //GNSS time
        Eigen::Vector3d linear;
        Eigen::Vector3d angular;
    };

    void UAVGlobalPosCallback(const sensor_msgs::NavSatFix::Ptr& unav);
    void UAVOrientCallback(const sensor_msgs::Imu::Ptr& uori);
    void UAVVelocityCallback(const geometry_msgs::TwistStamped::Ptr& velMsg);
    void set_global_NED(CoordinateFrames NED){globalNED=NED;};

    void stateCallback(const mavros_msgs::State::Ptr& state);
    bool isArmed();

    //Time functions
    void UAV_timeCallback(const sensor_msgs::TimeReference::Ptr& timeMsg);
    bool hasDt();
    ros::Time get_GNSS_time();
    ros::Duration get_dtGnssRos();


    position get_uav_pos();
    orientation get_uav_orient();
    velocity get_uav_velocity();

private:
    CoordinateFrames& globalNED;
    kalmanFilter& kf;
    CoordinateFrames::geodetic UAVgeodetic;

    position uav_pos;
    orientation uav_ori;
    velocity uav_vel;

    bool debug = false;
    bool isArmed_=false;

    //Time variables
    bool m_hasDt;
    ros::Duration dtGnssRos;
    ros::Time gnssTime;

};




#endif //PROJECT_UAVCALLBACK_H
