//
// Created by Line on 21.03.18.
//
#include <utils/readRosParam.h>
#include "arucoCallback.h"

ArucoCallback::ArucoCallback(kalmanFilter& kf_):kf{kf_}{
    //Read parameters from parameter file
    R_cu=ReadRosParam::readParameterMatrix("systemParameters/rotationCamBodyUAV");
    enableWriteToFile = ReadRosParam::readParameterBool("systemParameters/saveToFIle");
    varianceUAVorient = ReadRosParam::readParameterDouble("filterParameters/varianceUAVOrient");
    varianceArUcoPos_xy = ReadRosParam::readParameterDouble("filterParameters/varianceArucoPos/xy");
    varianceArUcoPos_height = ReadRosParam::readParameterDouble("filterParameters/varianceArucoPos/height");

    if(enableWriteToFile){
        myfile.open("KfArUco.csv");
        wf.writeHeader(myfile,"KF_ArUco_Time;KF_ArUco_pos_NED");
    }
}

void ArucoCallback::poseArucoCallback(const node_msgs::arucoPose::ConstPtr& arucoData) {

    aruco_pose.timestamp=arucoData->header.stamp+dtGnssRos;
    //aruco_pose.timestamp=arucoData->header.stamp;
    Eigen::Vector3d posRelCam(arucoData->pose.position.x, arucoData->pose.position.y, arucoData->pose.position.z);
    aruco_pose.position = posRelCam;

    //Transform to euler
    Eigen::Quaterniond orient(arucoData->pose.orientation.w,arucoData->pose.orientation.x,arucoData->pose.orientation.y,arucoData->pose.orientation.z);
    //Todo: Transform to uav frame!
    aruco_pose.orientation=CoordinateFrames::ENU2NED_euler(orient.toRotationMatrix().eulerAngles(0,1,2));

    if(kf.uav_orient_available())
    {
        //Finds the corresponding measurement from UAV_orientation that matches the time stamp
        kalmanFilter::bufferElement temp_element = kf.get_UAV_orient(aruco_pose.timestamp);

        //Normalize the quaternion
        Eigen::Quaterniond norm_quaternion(temp_element.element.w(),temp_element.element.x(),temp_element.element.y(),temp_element.element.z());
        norm_quaternion.normalize();

        //Get the rotation matrix from UAV frame to NED
        Eigen::Matrix3d R_un = norm_quaternion.toRotationMatrix();

        Eigen::Vector3d aruco_pos_NED = R_un*R_cu*posRelCam;

        //Update the covariance matrix
        Eigen::Matrix3d covar_pos = calculateCovariance(aruco_pos_NED.z());

        //Update Kalman filter with measurement
        ros::Time time_now = ros::Time::now();
        kf.updateMeasurement(kalmanFilter::measurementType::arUco_pos,aruco_pos_NED,aruco_pose.timestamp,covar_pos);
        //Write to file
        std::list<Eigen::MatrixXd> my_data = {aruco_pos_NED};
        if(enableWriteToFile)wf.writeDataToFile(myfile,aruco_pose.timestamp,my_data);

    }
}

ArucoCallback::pose ArucoCallback::get_aruco_pose() {
    return aruco_pose;
}

Eigen::Matrix3d ArucoCallback::calculateCovariance(double height) {
    Eigen::Vector3d variance(sin(varianceUAVorient)*height+varianceArUcoPos_xy,
                             sin(varianceUAVorient)*height+varianceArUcoPos_xy,varianceArUcoPos_height);
    Eigen::Matrix3d covar;
    covar << sqrt(variance.x()),0,0,0,sqrt(variance.y()),0,0,0,sqrt(variance.z());
    return covar;
}

void ArucoCallback::setDtGnssRos(ros::Duration dtGnssRos_) {
    dtGnssRos=dtGnssRos_;
}
