//
// Created by line on 14.02.18.
//
#include <node_navigation/tools/coordinateFrames.h>
#include <iostream>
#include <Eigen/Geometry>


CoordinateFrames::CoordinateFrames():NEDdefined{false} {
    geodetic initNed;
    initNed.latitude=63.418585;
    initNed.longitude=10.402820;
    initNed.altitude=48.00;
    NED=initNed;
}

CoordinateFrames::CoordinateFrames(const geodetic& ned):NEDdefined{true}{
    NED=ned;
}

// Converts geodetic to ECEF (x,y,z)
Eigen::Vector3d CoordinateFrames::geodetic2ECEF(const geodetic& geo)
{
    // Earth constants (WGS84)
    //Semi-major axis
    static const double R_Ea = 6378137.0;     //m
    //Flattering factor
    static const double f = 1/298.257223563;
    //Semi minor axis
    static const double R_Eb = R_Ea*(1-f);    //m
    //First eccentricity
    static const double e = std::sqrt(std::pow(R_Ea,2.0)-std::pow(R_Eb,2.0))/R_Ea;
    // Prime vertical radius of curvature N_E
    const double N_E=R_Ea/sqrt(1-pow(e,2.0)*pow(sin(r2d*geo.latitude),2.0));

    Eigen::Vector3d ECEF;

    ECEF << (N_E+geo.altitude)*cos(r2d*geo.latitude)*cos(r2d*geo.longitude),(N_E+geo.altitude)*cos(r2d*geo.latitude)*sin(r2d*geo.longitude),(N_E*(1-pow(e,2.0))+geo.altitude)*sin(r2d*geo.latitude);

    return ECEF;
}


// Returns a vector from the refPoint to the destPoint given as (x,y,z)
Eigen::Vector3d CoordinateFrames::geodetic2vect(const geodetic& refPont, const geodetic& destPoint) {
    Eigen::Matrix3d R_n_e;  // Rotation matrix from ECEF to refPont
    R_n_e <<    -sin(r2d*refPont.latitude)*cos(r2d*refPont.longitude),-sin(r2d*refPont.latitude)*sin(r2d*refPont.longitude),cos(r2d*refPont.latitude),
                -sin(r2d*refPont.longitude),cos(r2d*refPont.longitude),0.0,
                -cos(r2d*refPont.latitude)*cos(r2d*refPont.longitude),-cos(r2d*refPont.latitude)*sin(r2d*refPont.longitude),-sin(r2d*refPont.latitude);
    //Debug
    //std::cout << "R_n_e: " << R_n_e << std::endl;
    return R_n_e*(geodetic2ECEF(destPoint)-geodetic2ECEF(refPont));
}


// Returns a vector from the NED frame to the point (x,y,z)
Eigen::Vector3d CoordinateFrames::getVectNED2geodetic(const geodetic& point){
    return geodetic2vect(NED,point);
}


Eigen::Vector3d CoordinateFrames::ENU2NED_euler(const Eigen::Vector3d& inVectEul)
{
    Eigen::Matrix3d RotMat;
    RotMat<<0.0,1.0,0.0,
            1.0,0.0,0.0,
            0.0,0.0,-1.0;

    return RotMat*inVectEul;
}

Eigen::Quaterniond CoordinateFrames::ENU2NED_quat(const Eigen::Quaterniond& quat_ENU) {
    Eigen::Quaterniond quat_NED(quat_ENU.w(),quat_ENU.y(),quat_ENU.x(),-quat_ENU.z());

    return quat_NED;
}