//
// Created by Line on 20.03.18.
//

#include "node_navigation/tools/uavCallback.h"


UavCallback::UavCallback(CoordinateFrames& NED, kalmanFilter& kf_)
        :globalNED{NED},kf{kf_},m_hasDt{false}{
}

// Reading the /mavros/global_position/global from the UAV
void UavCallback::UAVGlobalPosCallback(const sensor_msgs::NavSatFix::Ptr& unav)
{
    // Get the relevant data from the UAV, lat, long, alt
    UAVgeodetic.latitude=unav->latitude;
    UAVgeodetic.longitude=unav->longitude;
    UAVgeodetic.altitude=unav->altitude;

    //Get the timing data and convert to GNSS time
    uav_pos.timestamp = unav->header.stamp+dtGnssRos;

    //Check if there is defined a global NED
    if (!globalNED.isNedDefined())
    {
        globalNED.setNED(UAVgeodetic);

        ROS_INFO("NAVIGATION: Global NED set to lat:%f long:%f alt:%f",UAVgeodetic.latitude,UAVgeodetic.longitude,UAVgeodetic.altitude);
    }

    // Transform relative to the global NED frame
    uav_pos.position = globalNED.getVectNED2geodetic(UAVgeodetic);

    // Debug
    if (debug) std::cout << "UAV relative to NED: " << std::endl << uav_pos.position << std::endl;

    // Read out the pose covariance matrix
    uav_pos.covariance=Eigen::Map<Eigen::MatrixXd>(unav->position_covariance.data(),3,3);

    // Store the measurement in buffer
    kf.set_UAV_position(uav_pos.position,uav_pos.timestamp,uav_pos.covariance.diagonal());
}

// Reading the /mavros/imu/data from the UAV
void UavCallback::UAVOrientCallback(const sensor_msgs::Imu::Ptr& uori)
{
    // Get orientation data from the UAV (quaternions)
    Eigen::Quaterniond orientation_ENU(uori->orientation.w,uori->orientation.x,uori->orientation.y,uori->orientation.z);

    // Convert the orientation vector from ENU to NED
    uav_ori.orientation=CoordinateFrames::ENU2NED_quat(orientation_ENU);

    //Get the timing data and convert to GNSS time
    uav_ori.timestamp = uori->header.stamp+dtGnssRos;

    // Store the measurement in buffer
    Eigen::Vector3d temp_covariance(0,0,0);
    kf.set_UAV_orient(uav_ori.orientation,uav_ori.timestamp,temp_covariance);
}

void UavCallback::UAV_timeCallback(const sensor_msgs::TimeReference::Ptr& timeMsg){
    dtGnssRos = timeMsg->time_ref-timeMsg->header.stamp;
    dtGnssRos =timeMsg->time_ref-ros::Time::now();
    gnssTime = timeMsg->time_ref;

    //ROS_INFO("NAVIGATION: dtGnssRos is set to %i s and %i ns", dtGnssRos.sec, dtGnssRos.nsec);

    m_hasDt = true;
}

void UavCallback::stateCallback(const mavros_msgs::State::Ptr &state) {
    isArmed_=state->armed;
}

bool UavCallback::isArmed() {
    return isArmed_;
}

void UavCallback::UAVVelocityCallback(const geometry_msgs::TwistStamped::Ptr& velMsg){
    uav_vel.timestamp = velMsg->header.stamp+dtGnssRos;

    Eigen::Vector3d linear_vel_ENU(velMsg->twist.linear.x,velMsg->twist.linear.y,velMsg->twist.linear.z);
    Eigen::Vector3d angular_vel_ENU(velMsg->twist.angular.x,velMsg->twist.angular.y,velMsg->twist.angular.z);

    //Rotate from ENU to NED
    uav_vel.linear = CoordinateFrames::ENU2NED_euler(linear_vel_ENU);
    uav_vel.angular =CoordinateFrames::ENU2NED_euler(angular_vel_ENU);  // Todo: Sjekk ut om dette er riktig?

    //Update the Kalman filter
    kf.set_u(uav_vel.linear);
}

UavCallback::position UavCallback::get_uav_pos() {
    return uav_pos;
}

UavCallback::orientation UavCallback::get_uav_orient() {
    return uav_ori;
}

UavCallback::velocity UavCallback::get_uav_velocity() {
    return uav_vel;
}

ros::Duration UavCallback::get_dtGnssRos(){
    return dtGnssRos;
}

ros::Time UavCallback::get_GNSS_time(){
    return gnssTime;
}


bool UavCallback::hasDt(){
    return m_hasDt;
}