#include <readRosParam.h>

Eigen::MatrixXd ReadRosParam::readParameterMatrix(std::string parameterName){
    //Returns a 1x1 matrix with with 0 as element if error.
    Eigen::MatrixXd tempMatrix;
    std::vector<float> tempVector;
    ros::NodeHandle parameterNode;
    int rows, cols;

    if(parameterNode.getParam("/"+parameterName+"/data",tempVector))
    {
        parameterNode.getParam("/" + parameterName + "/rows", rows);
        parameterNode.getParam("/" + parameterName + "/cols", cols);
        tempMatrix.resize(rows, cols);
        for (int i = 0; i < rows * cols; i++) {
            tempMatrix(int(floor(i / cols)), i % cols) = tempVector[i];
        }
    }
    else
    {
        ROS_WARN("Error loading parameter %s!",parameterName.c_str());
        tempMatrix=Eigen::MatrixXd::Zero(1,1);
    }
    return tempMatrix;
}

bool ReadRosParam::readParameterBool(std::string parameterName){
    bool tempBool;
    ros::NodeHandle parameterNode;
    if(parameterNode.getParam("/"+parameterName,tempBool))
    {
        return tempBool;
    }
    else
    {
        ROS_WARN("Error loading parameter %s!",parameterName.c_str());
        return false;
    }
}

double ReadRosParam::readParameterDouble(std::string parameterName) {
    double tempDouble;
    ros::NodeHandle parameterNode;
    if(parameterNode.getParam("/"+parameterName,tempDouble))
    {
        return tempDouble;
    }
    else
    {
        ROS_WARN("Error loading parameter %s!",parameterName.c_str());
        return 0.0;
    }
}